from scrappy.driver.worker import Worker
from scrappy.driver.pool import create, dispose
from scrappy.tasks.task import Task
from itertools import cycle
from threading import Thread
from queue import Queue
import pydebug

debug = pydebug.debug("scheduler")


class Scheduler(Thread):

    def __init__(self, pool_size, headless=True):
        Thread.__init__(self)
        self.pool = list()
        self.retry = Queue()
        self.tasks = Queue()
        self.pool_size = pool_size
        self.headless = headless
        self.pool = create(pool_size, headless)

    def __enter__(self):
        return self

    def __exit__(self, *arg):
        self.start()
        dispose(self.pool)

    def schedule_task(self, task):
        """Enqueues a task
        """
        if(not issubclass(task.__class__, Task)):
            raise ValueError("task must be an instance of Task")
        self.tasks.put(task)

    def consume_queue(self):
        raise NotImplementedError

    def dispose(self):
        """Disposes of all workers
        """
        dispose(self.pool)

    def run(self):
        self.consume_queue(self.tasks)
        while(self.retry._qsize() > 0):
            self.consume_queue(self.retry)



from scrappy import app
from scrappy.core.crash_dump import crash_dump
from flask import jsonify, request
import pydebug

debug = pydebug.debug("distributed:crash_dump")


@app.route("/crash_dump", methods=['POST'])
def crash_dump_route():
    data = request.get_json(force=True)
    try:
        crash_dump(data['caller_id'], data['error'])
        return '', 200
    except Exception as error:
        debug("Error while trying to write crash dump: {}".format(str(error)))
        return '', 400



from scrappy.tasks.task import Task
from scrappy.persistor.dummy_persistor import DummyPersistor
from scrappy.persistor.document import Document


class SimpleTask(Task):
    def __init__(self, persistor):
        Task.__init__(self, persistor)

    def run(self, driver, persistor):
        driver.get("http://google.com")
        driver.find_element_by_xpath(
            "//input[@title='Pesquisar']").send_keys("Hello world")

        doc = Document(1, "The quick brown fox jumps over the lazy dog")
        persistor.save_one(persistor, document=doc)


HelloGoogle = SimpleTask(DummyPersistor)



from scrappy.tasks.task import Task


class SimpleTask(Task):
    def __init__(self, persistor):
        Task.__init__(self, persistor)

    def run(self, driver, persistor):
        raise NotImplementedError



from scrappy.driver.worker import Worker
import time
from scrappy.driver.pool import create
from itertools import cycle
from threading import Thread
from scrappy.scheduler.scheduler import Scheduler
from queue import Queue
import pydebug

debug = pydebug.debug("scheduler")


class RoundRobin(Scheduler):
    def __init__(self, pool_size, headless=True):
        super().__init__(pool_size, headless)

    def consume_queue(self, q):
        """task queue consumption implementation

        Distributes tasks in a Robin Round fashion

        Arguments:
            q {[type]} -- [description]
        """
        gen = cycle(range(self.pool_size))
        for _ in range(q.qsize()):
            self.pool[next(gen)].schedule_task(self.tasks.get())

        for worker in self.pool:
            worker.start()



from time import gmtime, strftime
from scrappy.util.tmpFile import tmpFile
from uuid import uuid4
import pydebug

debug = pydebug.debug("core:logger")


def format_error(error):
    """
        format_error(error) => (string, integer)

        Formats an error message, returning formated output and generated id

        Keyword arguments:

    """

    time = strftime("%Y-%m-%d-%H:%M:%S", gmtime())
    uuid = uuid4()
    error = '---\nid: {id}\ntime: {time}\nmessage: "{msg}"\n---'.format(
        id=uuid, time=time, msg=str(error))

    return (error, uuid)


def crash_dump(caller_id, error, path=tmpFile("scrappy.dump")):
    """
        Securely logs an error

        Keyword arguments:
            caller_id  -- id of the caller
            error -- error to be logged
            path  -- path to log file (default tmpFile("scrappy.dump"))
    """

    formatedError, error_id = format_error(error)

    with open(path, "a") as file:
        msg = "{id} has crashed, logging with error_id {error_id}:\
               {error_summary}...".format(id=caller_id, error_id=error_id,
                                          error_summary=formatedError[:20])
        debug(msg)
        file.write(str(formatedError))



import setuptools

long_description = ""

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="scrappy_webdriver",
    version="0.0.2",
    author="V-in",
    author_email="viniciusmoura@cc.ci.ufpb.br",
    description="Lightweight distributed scrapper for dynamic content",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/V-in/scrappy",
    install_requires=[
        'atomicwrites',
        'attrs',
        'autopep8',
        'Click',
        'Flask',
        'itsdangerous',
        'Jinja2',
        'MarkupSafe',
        'more-itertools',
        'pep8',
        'pluggy',
        'py',
        'pycodestyle',
        'pydebug',
        'pytest',
        'selenium',
        'six',
        'urllib3',
        'Werkzeug',

    ],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
1



from contextlib import contextmanager
from scrappy.core import crash_dump
from queue import Empty, Queue
from selenium import webdriver
from threading import Thread, Event
from uuid import uuid4
from scrappy.core.commands import Die
import time
import pydebug

debug = pydebug.debug("driver:worker")


class Worker(Thread):
    """Scrapper worker
    A Worker continuously consumes and executes tasks from its internal queue
    until a Die task arrives.
    """

    def __init__(self, headless=True):
        Thread.__init__(self)
        self.driver = None
        self.headless = headless
        self._id = uuid4()
        self.queue = Queue()
        self.is_open = False

    def __del__(self):
        self.dispose()

    def __enter__(self):
        return self

    def __exit__(self, *a):
        self.schedule_task(Die)

    def _open(self):
        """Instanciates a driver window and sets internal .is_open flag to True
        """

        self.is_open = True
        options = webdriver.ChromeOptions()
        if(self.headless):
            options.add_argument('headless')
        self.driver = webdriver.Chrome(
            "chromedriver", options=options)
        debug("Worker {} has started".format(self._id))

    def dispose(self):
        """Disposes of active driver windowsd
        """
        try:
            self.driver.quit()
        except:
            pass

    def schedule_task(self, task):
        """Enqueues a task to be executed later

        Arguments:
            task task -- task to be enqueued
        """
        self.queue.put(task)

    def run(self):
        """Worker loop
        """

        self._open()

        while True:
            task = self.queue.get()
            if task is Die:
                self.queue.task_done()
                break
            try:
                task._run(self.driver)
            except Exception as error:
                debug(error)
                crash_dump.crash_dump(self._id, error)
            self.queue.task_done()

        self.dispose()
        debug("Worker {} has stopped".format(self._id))



from scrappy.driver.worker import Worker
from scrappy.core.commands import Die
import time


def create(size, headless=True):
    return list([Worker(headless) for _ in range(size)])


def start_all(pool):
    """Starts all workers in a pool

    Arguments:
        pool Pool -- Pool of workers
    """
    for worker in pool:
        worker.start()


def dispose(pool):
    """Disposes of a pool of workers

    Arguments:
        pool Pool -- Pool of workers
    """
    for worker in pool:
        worker.schedule_task(Die)
    for worker in pool:
        worker.queue.join()



class Task():
    def __init__(self, persistor):
        self.persistor = persistor

    def bootstrap_condition(self, driver):
        return True

    def bootstrap(self, driver):
        pass

    def _run(self, driver):
        if(self.bootstrap_condition(driver)):
            self.bootstrap(driver)
        self.run(driver, self.persistor)
